@@ -1,5 +1,5 @@
 /*
- * $Id: dq_string.c 3150 2010-04-08 11:36:13Z jakob $
+ * $Id: dq_string.c 4294 2011-01-13 19:58:29Z jakob $
  *
  * Copyright (c) 2008-2009 Nominet UK. All rights reserved.
  *
@@ -177,6 +177,10 @@ static void DqsAppendComparison(char** query, DQS_COMPARISON compare)
         StrAppend(query, " IN ");
         break;
 
+    case DQS_COMPARE_NOT_IN:
+        StrAppend(query, " NOT IN ");
+        break;
+
     case DQS_COMPARE_IS:
         StrAppend(query, " IS ");
         break;
