@@ -1,5 +1,5 @@
 /*
- * $Id: db_fields.h 3150 2010-04-08 11:36:13Z jakob $
+ * $Id: db_fields.h 4294 2011-01-13 19:58:29Z jakob $
  *
  * Copyright (c) 2008-2009 Nominet UK. All rights reserved.
  *
@@ -97,10 +97,13 @@ extern "C" {
 #define DB_SECURITY_MODULE_REQUIREBACKUP 3
 
 #define DB_ZONE_TABLE			"zones"
-#define DB_ZONE_FIELDS 			"id, name"
+#define DB_ZONE_FIELDS 			"id, name, policy_id, signconf, input, output"
 #define DB_ZONE_ID				0
 #define DB_ZONE_NAME			1
 #define DB_ZONE_POLICY_ID   	2
+#define DB_ZONE_SIGNCONF       	3
+#define DB_ZONE_INPUT       	4
+#define DB_ZONE_OUTPUT       	5
 
 /* policy select variables, including salt */
 #define DB_POLICY_ID	0
