@@ -1,5 +1,5 @@
 /*
- * $Id: ksm.h 3150 2010-04-08 11:36:13Z jakob $
+ * $Id: ksm.h 4294 2011-01-13 19:58:29Z jakob $
  *
  * Copyright (c) 2008-2009 Nominet UK. All rights reserved.
  *
@@ -57,6 +57,7 @@ int KsmInit(void);
 int KsmRundown(void);
 
 #define KSM_NAME_LENGTH     256         /* Includes trailing NULL */
+#define KSM_PATH_LENGTH     4096        /* Includes trailing NULL */
 #define KSM_POLICY_DESC_LENGTH     256  /* Includes trailing NULL */
 #define KSM_POLICY_AUDIT_LENGTH    4096 /* Includes trailing NULL */
 #define KSM_TIME_LENGTH     32          /* Includes trailing NULL */
@@ -126,8 +127,8 @@ typedef struct {
 } KSM_KEYDATA;
 
 int KsmKeyPairCreate(int policy_id, const char* HSMKeyID, int smID, int size, int alg, const char* generate, DB_ID* id);
-int KsmDnssecKeyCreate(int zone_id, int keypair_id, int keytype, DB_ID* id);
-int KsmDnssecKeyCreateOnPolicy(int policy_id, int keypair_id, int keytype);
+int KsmDnssecKeyCreate(int zone_id, int keypair_id, int keytype, int state, const char* time, DB_ID* id);
+/*int KsmDnssecKeyCreateOnPolicy(int policy_id, int keypair_id, int keytype);*/
 int KsmKeyInitSql(DB_RESULT* result, const char* sql);
 int KsmKeyInit(DB_RESULT* result, DQS_QUERY_CONDITION* condition);
 int KsmKeyInitId(DB_RESULT* result, DB_ID id);
@@ -135,11 +136,10 @@ int KsmKey(DB_RESULT result, KSM_KEYDATA* data);
 void KsmKeyEnd(DB_RESULT result);
 int KsmKeyQuery(const char* sql, DB_RESULT* result);
 int KsmKeyData(DB_ID id, KSM_KEYDATA* data);
-int KsmKeyPredict(int policy_id, int keytype, int shared_keys, int interval, int *count, int rollover_scheme);
+int KsmKeyPredict(int policy_id, int keytype, int shared_keys, int interval, int *count, int rollover_scheme, int zone_count);
 int KsmKeyCountQueue(int keytype, int* count, int zone_id);
 int KsmKeyCountStillGood(int policy_id, int sm, int bits, int algorithm, int interval, const char* datetime, int *count, int keytype);
-int KsmKeyGetUnallocated(int policy_id, int sm, int bits, int algorithm, int *keypair_id);
-int KsmLinkKeys(const char* zone_name, int policy_id);
+int KsmKeyGetUnallocated(int policy_id, int sm, int bits, int algorithm, int zone_id, int share_keys, int *keypair_id);
 int KsmMarkKeysAsDead(int zone_id);
 int KsmKillKey(int keypair_id);
 
@@ -148,10 +148,6 @@ int KsmKillKey(int keypair_id);
 int KsmDeleteKeyRange(int minid, int maxid);
 int KsmDeleteKeyRanges(int limit[], int size);
 
-/* modify */
-
-int KsmKeyModify(KSM_KEYDATA* data, int low, int high);
-
 /* KsmParameter */
 
 typedef struct {
@@ -249,7 +245,7 @@ typedef struct {
 
 typedef struct {
 	int id;
-	char* name;
+    char        name[KSM_NAME_LENGTH];
 	char* description;
 	KSM_SIGNER_POLICY* signer;
 	KSM_SIGNATURE_POLICY* signature;
@@ -280,6 +276,7 @@ int KsmPolicyParameter(DB_RESULT handle, KSM_POLICY_PARAMETER* data);
 int KsmPolicyReadFromId(KSM_POLICY* policy);
 int KsmPolicyNameFromId(KSM_POLICY* policy);
 int KsmPolicyUpdateSalt(KSM_POLICY* policy);
+int KsmPolicyNullSaltStamp(int policy_id);
 int KsmPolicyPopulateSMFromIds(KSM_POLICY* policy);
 int KsmPolicySetIdFromName(KSM_POLICY *policy);
 int KsmPolicyIdFromZoneId(int zone_id, int* policy_id);
@@ -289,8 +286,13 @@ void KsmPolicyFree(KSM_POLICY *policy);
 
 /* ksmZone */
 typedef struct {
-    int id;
-    char* name;
+    int   id;
+    int   policy_id;
+    char  name[KSM_ZONE_NAME_LENGTH];
+    char  signconf[KSM_PATH_LENGTH];
+    char  input[KSM_PATH_LENGTH];
+    char  output[KSM_PATH_LENGTH];
+    char  policy_name[KSM_NAME_LENGTH];
 } KSM_ZONE;
 
 int KsmZoneInit(DB_RESULT* handle, int policy_id);
@@ -530,17 +532,17 @@ int KsmKeywordParameterExists(const char* name);
 /* ksm_update */
 
 int KsmUpdate(int policy_id, int zone_id);
-void KsmUpdateKey(KSM_KEYDATA* data, KSM_PARCOLL* collection);
+void KsmUpdateKey(KSM_KEYDATA* data, KSM_PARCOLL* collection, int zone_id);
 void KsmUpdateGenerateKeyTime(KSM_KEYDATA* data);
-void KsmUpdatePublishKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection);
+void KsmUpdatePublishKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection, int zone_id);
 void KsmUpdateReadyKeyTime(KSM_KEYDATA* data);
-void KsmUpdateActiveKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection);
-void KsmUpdateRetireKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection);
+void KsmUpdateActiveKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection, int zone_id);
+void KsmUpdateRetireKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection, int zone_id);
 void KsmUpdateDeadKeyTime(KSM_KEYDATA* data);
-void KsmUpdateDSPublishKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection);
-void KsmUpdateKEYPublishKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection);
+void KsmUpdateDSPublishKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection, int zone_id);
+void KsmUpdateKEYPublishKeyTime(KSM_KEYDATA* data, KSM_PARCOLL* collection, int zone_id);
 int KsmUpdateKeyTime(const KSM_KEYDATA* data, const char* source,
-    const char* destination, int interval);
+    const char* destination, int interval, int zone_id);
 
 /* ksm_request */
 
@@ -583,7 +585,7 @@ int KsmRequestStandbyKSKCount(int* count, int zone_id);
 int KsmRequestCheckActiveKey(int keytype, const char* datetime, int* count, int zone_id);
 int KsmRequestCountReadyKey(int keytype, const char* datetime, int* count, int zone_id);
 int KsmRequestCheckFirstPass(int keytype, int* first_pass_flag, int zone_id);
-
+int KsmRequestCheckCompromisedFlag(int keytype, int zone_id, int* comp_flag);
 int KsmRequestIssueKeys(int keytype, KSM_REQUEST_CALLBACK callback,
 	void* context, int zone_id);
 
@@ -610,17 +612,19 @@ int KsmPolicyInitialPublicationInterval(KSM_POLICY *policy);
 /* KsmImport */
 int KsmImportRepository(const char* repo_name, const char* repo_capacity, int require_backup);
 int KsmImportPolicy(const char* policy_name, const char* policy_description);
-int KsmImportZone(const char* zone_name, int policy_id, int fail_if_exists, int *new_zone);
+int KsmImportZone(const char* zone_name, int policy_id, int fail_if_exists, int *new_zone, const char* signconf, const char* input, const char* output);
 int KsmImportAudit(int policy_id, const char* audit_contents);
-int KsmImportKeyPair(int policy_id, const char* HSMKeyID, int smID, int size, int alg, int state, const char* time, const char* opt_time, DB_ID* id);
+int KsmImportKeyPair(int policy_id, const char* HSMKeyID, int smID, int size, int alg, int state, const char* time, DB_ID* id);
 int KsmSmIdFromName(const char* name, int *id);
 int KsmSerialIdFromName(const char* name, int *id);
 int KsmPolicyIdFromName(const char* name, int *id);
+int KsmMarkPreBackup(int repo_id, const char* datetime);
+int KsmRollbackMarkPreBackup(int repo_id);
 int KsmMarkBackup(int repo_id, const char* datetime);
 int KsmCheckHSMkeyID(int repo_id, const char* cka_id, int *exists);
 
 /* KsmList */
-int KsmListBackups(int repo_id);
+int KsmListBackups(int repo_id, int verbose_flag);
 int KsmListRepos();
 int KsmListPolicies();
 int KsmListRollovers(int zone_id);
